module Perfimmo.Form.FormField.FormFloatTest exposing (..)

import Perfimmo.Form.FormField.FormFloat as FormFloat
import Test exposing (..)
import Expect

suite = describe "FormFloat Test"
    [ describe "FormFloat.build"
        [ test "test FormFloat.build with empty String" <| \_ ->
            let result = FormFloat.setValueFromS "" FormFloat.empty
            in result |> FormFloat.toFloat |> Expect.equal Nothing

        , test "test FormFloat.build with 42" <| \_ ->
            let result = FormFloat.setValueFromS "42" FormFloat.empty |> FormFloat.toFloat
                expected = Just <| 42
            in result |> Expect.equal expected

        , test "test FormFloat.build with 42.3" <| \_ ->
            let result = FormFloat.setValueFromS "42.3" FormFloat.empty |> FormFloat.toFloat
                expected = Just <| 42.3
            in result |> Expect.equal expected

        , test "'42.' is a valid input for an user input in elm form" <| \_ ->
            let result = FormFloat.setValueFromS "42." FormFloat.empty |> FormFloat.toFloat
                expected = Just <| 42
            in result |> Expect.equal expected

        , test "'42.s' is an invalid input" <| \_ ->
            let result = FormFloat.setValueFromS "42.s" FormFloat.empty
                expected = Nothing
            in result |> FormFloat.toFloat |> Expect.equal expected

        , test "'42.2.1' is an invalid input" <| \_ ->
            let result = FormFloat.setValueFromS "42.2.1" FormFloat.empty
                expected = Nothing
            in result |> FormFloat.toFloat |> Expect.equal expected

        , test "'42..' is an invalid input" <| \_ ->
            let result = FormFloat.setValueFromS "42.." FormFloat.empty
                expected = Nothing
            in result |> FormFloat.toFloat |> Expect.equal expected

        , test "'42,2' is a valid input for an user input in elm form" <| \_ ->
            let result = FormFloat.setValueFromS "42,2" FormFloat.empty |> FormFloat.toFloat
                expected = Just <| 42.2
            in result |> Expect.equal expected

        , test "'42,' is a valid input for an user input in elm form" <| \_ ->
            let result = FormFloat.setValueFromS "42," FormFloat.empty |> FormFloat.toFloat
                expected = Just <| 42
            in result |> Expect.equal expected
        ]

    , describe "FormFloat.setOptionalValue"
        [ test "should remove previous value when setting Nothing" <| \_ ->
            let formField = FormFloat.empty |> FormFloat.setValue 12
                result = formField |> FormFloat.setOptionalValue Nothing
            in FormFloat.toFloat result |> Expect.equal Nothing

        , test "should set new value" <| \_ ->
            let formField = FormFloat.empty |> FormFloat.setValue 12
                result = formField |> FormFloat.setOptionalValue (Just 13)
            in FormFloat.toFloat result |> Expect.equal (Just 13)

        , test "should set new value when Nothing" <| \_ ->
            let formField = FormFloat.empty
                result = formField |> FormFloat.setOptionalValue (Just 13)
            in FormFloat.toFloat result |> Expect.equal (Just 13)
        ]

    , describe "FormFloat.toString"
        [ test "test FormFloat.toString with 42" <| \_ ->
            let result = FormFloat.setValueFromS "42" FormFloat.empty |> FormFloat.toString
                expected = "42"
            in result |> Expect.equal expected

        , test "test FormFloat.toString with 42.3" <| \_ ->
            let result = FormFloat.setValueFromS "42.3" FormFloat.empty |> FormFloat.toString
                expected = "42.3"
            in result |> Expect.equal expected

        , test "test FormFloat.toString with 42." <| \_ ->
            let result = FormFloat.setValueFromS "42." FormFloat.empty |> FormFloat.toString
                expected = "42."
            in result |> Expect.equal expected

        , test "test FormFloat.toString with 42,3" <| \_ ->
            let result = FormFloat.setValueFromS "42,3" FormFloat.empty |> FormFloat.toString
                expected = "42,3"
            in result |> Expect.equal expected

        , test "test FormFloat.toString with 42," <| \_ ->
            let result = FormFloat.setValueFromS "42," FormFloat.empty |> FormFloat.toString
                expected = "42,"
            in result |> Expect.equal expected
        ]

    , describe "FormFloat.fromFloat"
        [ test "test FormFloat.fromFloat with 42" <| \_ ->
             let result = FormFloat.setValue 42 FormFloat.empty |> FormFloat.toFloat
                 expected = Just 42
             in result |> Expect.equal expected

        ]

    , describe "FormFloat.add"
        [ test "test FormFloat.add 0.1 0.2" <| \_ ->
            -- https://0.30000000000000004.com/
            let f1 = FormFloat.setValue 0.1 FormFloat.empty
                f2 = FormFloat.setValue 0.2 FormFloat.empty
                result = FormFloat.add f1 f2 |> FormFloat.toFloat
            in result |> Expect.equal (Just 0.3)
        ]
    ]